Package: GOpro
Type: Package
Title: Find the most characteristic gene ontology terms for groups of
        human genes
Version: 1.39.0
Author: Lidia Chrabaszcz
Maintainer: Lidia Chrabaszcz <chrabaszcz.lidia@gmail.com>
Description: Find the most characteristic gene ontology terms for
    groups of human genes.
    This package was created as a part of
    the thesis which was developed under the auspices of MI^2 Group
    (http://mi2.mini.pw.edu.pl/, https://github.com/geneticsMiNIng).
License: GPL-3
Depends: R (>= 3.5.0)
Imports: AnnotationDbi, dendextend, doParallel, foreach, parallel,
        org.Hs.eg.db, GO.db, Rcpp, stats, graphics,
        MultiAssayExperiment, IRanges, S4Vectors
LinkingTo: Rcpp, BH
RoxygenNote: 6.0.1
Suggests: knitr, rmarkdown, RTCGA.PANCAN12, BiocStyle, testthat
VignetteBuilder: knitr
LazyData: true
biocViews: Annotation, Clustering, GO, GeneExpression,
        GeneSetEnrichment, MultipleComparison
URL: https://github.com/mi2-warsaw/GOpro
BugReports: https://github.com/mi2-warsaw/GOpro/issues
git_url: https://git.bioconductor.org/packages/GOpro
git_branch: devel
git_last_commit: 91eede1
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 22:57:41 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

export(findGO)
import(AnnotationDbi)
import(GO.db)
import(dendextend)
import(doParallel)
import(foreach)
import(org.Hs.eg.db)
import(parallel)
importFrom(IRanges,CharacterList)
importFrom(IRanges,IntegerList)
importFrom(IRanges,NumericList)
importFrom(MultiAssayExperiment,assay)
importFrom(MultiAssayExperiment,experiments)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,List)
importFrom(graphics,legend)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(stats,TukeyHSD)
importFrom(stats,aggregate)
importFrom(stats,aov)
importFrom(stats,as.dendrogram)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
useDynLib(GOpro)
useDynLib(GOpro, .registration=TRUE)

# GoPro

**A complete Python GoPro module made easy.**

![GoPro](http://cdn.snowboarding.transworld.net/wp-content/blogs.dir/442/files/2014/09/gopro-hero4-new-camera-black-silver-editions-600x342.png)

At the moment this module only works with the newest GoPro Hero 4.

Sadly the earlier versions of GoPro are not backwards/forwards compatible.



## Install

Easy install with Github or PyPi

Installing the **latest version from GitHub**.

```bash
$ git clone https://github.com/DenisCarriere/gopro.git
$ cd gopro
$ python setup.py install
```

Installing the **tested version from PyPi**.

```bash
$ sudo pip install gopro
```

## Quickstart

Once the installation is complete you are ready to start! :)

The default `IP` address for the GoPro is using `10.5.5.9`.

You must first be connected to the GoProApp Wifi connection.

Launch your favorite `ipython` or standard `python`

```python
>>> from gopro import GoPro
>>> camera = GoPro()
>>> camera.status
{'connection': 'OK',
 'datetime': '2014-11-12T23:53:59',
 'ip': '10.5.5.9',
 'ok': True,
 'screen': 'video',
 'storage': '31.3 GB',
 'time_offset': 'now'}
...
```

## Take a Photo or Video

With very simple commands you can tell your GoPro to start recording or take a photo.

The screen will automaticly change to the approriate page.

```python
>>> camera.photo()
>>> camera.video()
>>> camera.timelapse()
```

## Camera modes

Makes it easy to cycle threw each available modes.

```python
>>> camera.mode('burst')
>>> camera.mode('photo')
>>> camera.mode('timelapse')
>>> camera.mode('video')
...
```

## Locate (Beep! Beep!)

Can't find your device?? Call the locate function and it will Beep endlessly until there is no more battery.

To turn off the beeps, simply press the `mode` button on the device or use the `off` as a parameter.

```python
>>> camera.locate('on')
>>> camera.locate('off')
...
```

## Sleep

The camera will turn itself off, the Wi-Fi will still be active.

At the moment there is no working commands to turn the device back on.

To reactivate the device, press the `mode` button.

```python
>>> camera.sleep()
...
```

## Delete Photos

Might be useful to dump all those photos from your GoPro.

**Warning** once you delete photos/videos you cannot retrieve them back.

The `erase` function does the same as the delete all.

```python
>>> camera.erase()
>>> camera.delete_all()
>>> camera.delete_last()
...
```

## Settings

You can explore the different types of settings & status by getting the raw JSON response or selecting a specific pre-parsed attribute

### Processed attributes

**Status - Screen**

This will let you know which page your GP is currently active.

There are only 4 available outcomes.

```python
>>> camera.status_screen
'settings'
'video'
'photo'
'timelapse'
...
```

### Raw JSON Responses

The original response from the GoPro device.

```python
>>> camera.settings_raw
>>> camera.status_raw
...
```

## Date & Time

It is a good idea to sync your GoPro with the same clock as your computer.

An easy way to do this is use the `datetime` &  `time_offset` attribute.

Try to aim your GoPro to have less than 1 second offset.

The `time_offset` is measured in (+/-) seconds.

```python
>>> camera.datetime
'2014-11-12 21:53:09'
>>> camera.datetime_offset
-0.320142
>>> camera.status['offset']
'now'
...
```

## File Storage

Here is how you can retrieve the total file storage in (KB) or pretty human readeable format.

```python
>>> camera.status_storage
31330144
>>> camera.status['storage']
'31.3 GB'
```

## Debug

Want to explore & troubleshoot the device a bit, you can retrieve the following properties from your device.

```python
>>> camera.debug
>>> camera.info
>>> camera.commands
...
```

## Contributors

A big thanks to all the people who have helped contribute!

To be a contributor, please message me with requests on [Twitter](https://twitter.com/DenisCarriere)/[Github issues](https://github.com/DenisCarriere/gopro/issues)

- @[Denis Carriere](https://github.com/DenisCarriere)
- @[Konrad Iturbe](https://github.com/KonradIT)
- @[Jason Moiron](https://github.com/jmoiron)

## License

The MIT License (MIT)

Copyright (c) 2014-2015 Denis Carriere

## ----style, echo = FALSE, results = 'asis'------------------------------------
BiocStyle::markdown()

## ----eval = FALSE-------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly=TRUE))
#     install.packages("BiocManager")
# BiocManager::install("GOpro", dependencies = TRUE)

## ----eval=TRUE, results='hide', warning=FALSE, message=FALSE------------------
library(GOpro)

## ----cache=TRUE, message=FALSE, results='markup'------------------------------
exrtcga

## ----results='hold', message=FALSE, cache=TRUE--------------------------------
findGO(exrtcga)

## ----results='hold', message=FALSE, cache=TRUE--------------------------------
findGO(exrtcga, extend = TRUE)

## ----results='markup', message=FALSE, cache=TRUE, fig.keep='all', fig.show='hold', dev='png'----
findGO(exrtcga, topGO = 2, grouped = 'clustering')

## ----results='markup', message=FALSE, cache=TRUE, fig.keep='all', fig.show='hold', dev='png'----
findGO(exrtcga, topGO = 2, grouped = 'clustering', over.rep = TRUE)

#' GOpro: find the most characteristic gene ontology terms for groups of genes
#'
#' Based on the gene expressions find the structure somewhat
#' comparable to gene signature. From all given genes,
#' determine which are significantly different between sets.
#' These sets may relate to different health conditions of patients,
#' i.e. different types of cancer.
#' Then divide interesting genes into subsets.
#' Genes belong to a particular subset if they share the same feature.
#' There are two implemented methods that can be used to create
#' genes' subsets.
#' The first method is so-called all pairwise comparisons by
#' Tukey's procedure.
#' Genes that have the same profile (a result of all comparisons)
#' are assigned to one subset.
#' The second way of determining subsets is a method of
#' hierarchical clustering.
#' When all genes are divided into subsets, then
#' for each subset all relevant GO terms are searched for
#' in org.Hs.eg.db database.
#' Each found GO terms is tested using Fisher's test to find out
#' which of them are the most characteristic for the given subset of genes.
#'
#' @docType package
#' @name GOpro
#' @useDynLib GOpro
#' @import AnnotationDbi
#' @import dendextend
#' @import doParallel
#' @import foreach
#' @import parallel
#' @import org.Hs.eg.db
#' @import GO.db
#' @importFrom  Rcpp sourceCpp
#' @importFrom graphics legend plot text
#' @importFrom stats TukeyHSD aggregate aov
#'  as.dendrogram dist hclust na.omit p.adjust
#' @importFrom S4Vectors DataFrame
#' @importFrom IRanges CharacterList NumericList
#' @seealso {\link{findGO}}
NULL

#' Find all groups - rearrange the dendrogram:
#'  find which elements were merged in each step.
#' @noRd
#' @param hc An object of class hclust.
#'
#' @return An object of class \code{genecluster},
#'  a list of labels of elements that were merged in each step.
#'
#' @examples
#' clusteredGenes <- clustering(exanova,
#'  clust.metric = 'manhattan', clust.method = 'average')
#' unbundleCluster(clusteredGenes)


unbundleCluster <- function(hc) {
    roots <- hc$labels[hc$order]
    merged <- hc$merge
    leaves <- lapply(1:(length(hc$labels) - 1), function(i) {
        hc$labels[-ordering(i, merged)]
    })
    out <- list(groups = c(roots, leaves), hclust = hc)
    names(out[[1]]) <- paste0("G", seq_along(out[[1]]))
    class(out) <- append("genecluster", class(out))
    out
}

ordering <- function(x, merged) {
    if (merged[x, 1] < 0) {
        a <- merged[x, 1]
    } else {
        a <- ordering(merged[x, 1], merged)
    }
    if (merged[x, 2] < 0) {
        b <- merged[x, 2]
    } else {
        b <- ordering(merged[x, 2], merged)
    }
    c(a, b)
}

context("rtcga data")

test_that("rtcga object is valid", {
    expect_true(length(exrtcga) > 2)
    obj <- rownames(exrtcga)
    expect_identical(obj[[1]], obj[[2]])
    expect_identical(obj[[1]], obj[[3]])
})


context("aovTopTest")

g2 <- experiments(exrtcga)
groups <- sapply(g2, assay)

test_that("when top is missing or 0 expect warning", {
    expect_warning(aovTopTest(groups, 0))
})

test_that("empty list causes error", {
    expect_error(aovTopTest(list(), 10))
})

context("groupByTukey")

test_that("empty list causes error", {
    expect_error(groupByTukey(list()))
})

context("fisher_test")

test_that("fisher.test/stats and fisher_test/GOpro p-values are equal", {
    f_stats <- fisher.test(matrix(c(3, 1, 2, 7), nrow = 2), alternative = 'greater')[[1]]
    f_GOpro <- fisher_test(3, 1, 2, 7)
    expect_equal(f_stats, f_GOpro)
})

context("unbundleCluster")

test_that("the number of groups = 2*(number of rows)-1", {
    kk <- data.frame(a = c(1, 5, 1), b = c(1, 2, 1))
    rownames(kk) <- c('r1', 'r2', 'r3')
    expect_equal(length(unbundleCluster(hclust(dist(kk)))[[1]]), 2 * nrow(kk) - 1)
})

context("ontologyOver")

test_that("over-represented ontology is correct", {
    expect_identical(ontologyOver(data.frame(GROUP = 1, ONTOLOGY = 'BP')), 'BP')
})

#' Find the description of characteristic GO functions.
#' @noRd
#' @param TopGOs A list of vectors with significant GO IDs.
#'
#' @return A data frame with an extended description
#'  (term, definition, and ontology) of gene ontology terms.
#'
#' @examples
#' tukey.results <- groupByTukey(exanova, sig.level = 0.1)
#' # find all and then top GO terms
#' # geneUniverse is a universe of all genes
#' # of interest, i.e. all genes measured by microarray
#' geneUniverse <- rownames(exrtcga[[1]])
#' all.gos <- findAllGOs(geneUniverse, tukey.results, min = 8, max = 400)
#' top.gos <- findTopGOs(all.gos, sig.level = 0.1, top = 2)
#' extendGO(top.gos)
#'
#'
#' @import GO.db
#' @importFrom S4Vectors DataFrame
#' @importFrom IRanges IntegerList CharacterList

extendGO <- function(TopGOs) {
    group.size <- sapply(TopGOs, length)
    nulls <- which(unlist(lapply(TopGOs, is.na)) == TRUE)
    TopG <- vector("character", sum(group.size))
    TopG[nulls] <- NA
    TopG[-nulls] <- na.omit(unlist(lapply(TopGOs, names)))
    GO.descr <- lapply(TopG, function(GOID) {
        if (!is.na(GOID)) {
                                select(GO.db, keys = GOID,
                                  columns = c("TERM", "DEFINITION",
                                              "ONTOLOGY"),
                                  keytype = "GOID")
        } else {
            data.frame(cbind(GOID = NA, TERM = NA,
                             DEFINITION = NA, ONTOLOGY = NA))
        }
    })
    gnames <- seq_along(group.size)
    assignedFunctions <- data.frame(rep(gnames, group.size),
                                    matrix(unlist(GO.descr),
                                           ncol = 4, byrow = TRUE))
    names(assignedFunctions) <- c("GROUP", "GO ID",
                                  "TERM", "DEFINITION", "ONTOLOGY")
    assignedFunctions
}

extendoutput <- function(top.gos){
    ext <- extendGO(top.gos)
    DataFrame(GROUP = IntegerList(ext[, 1]),
                  GOID = CharacterList(ext[, 2]),
                  TERM = CharacterList(ext[, 3]),
                  DEFINITION = CharacterList(ext[, 4]),
                  ONTOLOGY = CharacterList(ext[, 5]))
}

#' Cluster all interesting (differentially expressed)
#'  genes using hierarchical clustering (based on all groups together).
#' @noRd
#' @param resultAOV A list of data frames of gene expressions
#'  (genes in rows, samples in columns).
#' @param clust.metric The method to calculate a distance measure
#'  used in hierarchical clustering, possible names: \code{"euclidean"},
#' \code{"maximum"}, \code{"manhattan"}, \code{"canberra"},
#'  \code{"binary"} or \code{"minkowski"}.
#' @param clust.method The agglomeration method of clustering genes.
#'  This should be one of: \code{"ward.D"},
#'  \code{"ward.D2"}, \code{"single"},
#'  \code{"complete"}, \code{"average" (= UPGMA)},
#'  \code{"mcquitty" (= WPGMA)}, \code{"median" (= WPGMC)}
#'  or \code{"centroid" (= UPGMC)}.
#' @param dist.matrix A matrix with calculated distances
#' to be used as a metric by
#' \code{hclust} function.
#' The default distance is \code{euclidean}.
#'
#' @return An object of class \code{hclust}.
#'
#' @examples
#' clustering(exanova, clust.metric = 'manhattan', clust.method = 'average')

clustering <- function(resultAOV, clust.metric = NULL,
                       clust.method = NULL, dist.matrix = NULL) {
    clust <- do.call(cbind, resultAOV)
    row.names(clust) <- row.names(resultAOV[[1]])

    if (is.null(dist.matrix)) {
        if (is.character(clust.metric) & (length(clust.metric) == 1)) {
            distance <- dist(clust, method = clust.metric)
        } else {
            distance <- dist(clust, method = "euclidean")
        }
    } else {
        distance <- dist.matrix
    }
    if (is.null(clust.method)) {
        clust.method <- "average"
    }
    cluster1 <- hclust(distance, method = clust.method)
    cluster1
}

#' Calculate frequencies of each ontology in each group (CC, BP, MF).
#' @noRd
#' @param extendedGO A data frame with a one column of group
#'  and another column with GO term (returned by \code{extendGO}).
#'
#' @return A data frame with the most frequently occurring
#'  ontologies in each group.
#'
#' @examples
#' tukey.results <- groupByTukey(exanova, sig.level = 0.1)
#' # find all and then top GO terms
#' # geneUniverse is a universe of all genes of interest,
#' # i.e. all genes measured by microarray
#' geneUniverse <- rownames(exrtcga[[1]])
#' all.gos <- findAllGOs(geneUniverse, tukey.results)
#' top.gos <- findTopGOs(all.gos, sig.level = 0.1, top = 3)
#' extended <- extendGO(top.gos)
#' ontologyOver(extended)
#'

ontologyOver <- function(extendedGO) {
    group <- extendedGO$GROUP
    ontology <- extendedGO$ONTOLOGY
    BP <- vector("numeric", length(unique(group)))
    CC <- vector("numeric", length(unique(group)))
    MF <- vector("numeric", length(unique(group)))
    no <- length(unique(group))
    BP <- sapply(seq_len(no), function(x) {
        sum(extendedGO[extendedGO$GROUP == x, "ONTOLOGY"] == "BP")
    })
    CC <- sapply(seq_len(no), function(x) {
        sum(extendedGO[extendedGO$GROUP == x, "ONTOLOGY"] == "CC")
    })
    MF <- sapply(seq_len(no), function(x) {
        sum(extendedGO[extendedGO$GROUP == x, "ONTOLOGY"] == "MF")
    })
    counted.onto <- cbind.data.frame(group = seq_len(no),
                                     onto = rep(c("BP", "CC", "MF"),
                                                each = no),
                                     counts = c(BP, CC, MF))
    counted.onto$counts[is.na(counted.onto$counts)] <- 0
    sapply(1:no, function(x) {
        three <- counted.onto[c(x, x + no, x + 2 * no), 3]
        if (all(three == 0)) {
            return("No function")
        }
        names(three) <- counted.onto[c(x, x + no, x + 2 * no), 2]
        names(which(three == max(three)))
    })
}

#' Find all ontology terms annotated with genes.
#' @noRd
#' @param geneUni A vector of names of all genes that
#'  were primarily included in the analysis,
#'  i.e. all genes measured by the microarray.
#' @param geneSel A list of groups of selected genes
#'  or an hclust object representing clustered genes.
#' @param min A minimum number of genes that a function needs
#'  to be represented by to be considered in the analysis.
#' @param max A maximum number of genes that a function needs
#'  to be represented by to be considered in the analysis.
#' @param parallel A logical value indicating if a task
#'  should be run on more than one core.
#'  The default is \code{FALSE}.
#' @param onto An ontology or ontologies to be searched for
#'  significant GO terms, at least one of 'MF' (molecular function),
#'  'BP' (biological process), 'CC' (cellular component).
#'
#' @return A list of lists with all gene ontology terms
#'  identifiers relevant to each group of genes and p-values for Fisher's test.
#'
#' @examples
#' tukey.results <- groupByTukey(exanova, 0.1)
#' #geneUniverse is a universe of all genes of interest
#' geneUniverse <- rownames(exrtcga[[1]])
#' findAllGOs(geneUniverse, tukey.results)
#'
#'
#' @import AnnotationDbi
#' @import org.Hs.eg.db
#' @import doParallel
#' @import foreach
#' @import parallel

findAllGOs <- function(geneUni, geneSel, onto = c("MF", "BP", "CC"),
                       min = 5, max = 500, parallel = FALSE) {
    stopifnot(is.list(geneSel) &
                  is.vector(geneUni) &
                  all(is.character(geneUni)))

    if (!all(onto %in% c("MF", "BP", "CC")))
        stop(paste(onto, "is not a valid ontology domain."))

    if (any(is.null(geneSel$merge) &
            is.null(geneSel$dist.method) &
            is.null(geneSel$order))) {
        cgroups <- geneSel
    } else {
        cgroups <- unbundleCluster(geneSel)[[1]]
    }
    iter <- length(cgroups)
    all.annot <- select(org.Hs.eg.db, keys = geneUni,
                        keytype = "SYMBOL", columns = "GO")
    all.annot <- all.annot[all.annot$ONTOLOGY %in% onto, ]

    if (nrow(all.annot) == 0)
        stop("No Gene Ontology terms in the given ontology domain.")
    go2gene.lst.p <- aggregate(SYMBOL ~ GO, all.annot, invisible)
    counted.genes <- unname(sapply(go2gene.lst.p$SYMBOL, length))
    names(counted.genes) <- go2gene.lst.p$GO

    go2gene.lst <- go2gene.lst.p[which(
        counted.genes >= min & counted.genes <= max), ]
    uniSize <- length(geneUni)

    allFisherTests <- function(i) {
        in.group <- cgroups[[i]]
        group.gos <- unique(na.omit(
            all.annot$GO[(all.annot$SYMBOL %in% in.group)]))
        len.in <- length(in.group)
        if (length(group.gos) == 0) {
            return(NA)
        }
        lapply(group.gos, function(group.go) {
            if (sum(go2gene.lst$GO == group.go) == 0) {
                return(NA)
            } else {
                this.genes <- unname(unlist(
                    # names of genes with this f
                    go2gene.lst[go2gene.lst[, 1] == group.go, 2]))
                # the number of all genes with this f
                withGO <- unname(counted.genes[group.go])

                pp <- sum(sapply(seq_along(in.group),
                                 function(x) in.group[x] == this.genes))
                #liczba genow w grupie o danej f
                pn <- withGO - pp
                #liczba pozostalych genow o danej funkcji
                np <- len.in - pp
                #liczba genow w grupie bez danej funkcji
                nn <- uniSize - pp - pn - np
                #liczba genow poza grupa bez danej f

                if (!is.na(sum(c(pp, pn, np, nn))) &
                    all(c(pp, pn, np, nn) >= 0)) {
                    result <- fisher_test(pp, pn, np, nn)
                } else {
                    return(NA)
                }
            }
            out_rec <- c(result, group.go, in.group)
            out_rec
        })
    }
    if (missing(parallel)) {
        parallel <- FALSE
    }
    z <- 0
    if (parallel == TRUE) {
        cores <- detectCores() - 1
        cl <- makeCluster(cores)
        registerDoParallel(cl)
        out <- foreach(z = seq_len(iter), .packages = "GOpro") %dopar%
            allFisherTests(z)
        names(out) <- names(cgroups)
        stopCluster(cl)
        out
    } else {
        out <- foreach(z = seq_len(iter)) %do%
            allFisherTests(z)
        names(out) <- names(cgroups)
        out
    }
    remove.NA <- lapply(out, function(x) which(is.na(x)))
    out2 <- lapply(names(remove.NA), function(x) {
        out[[x]][-remove.NA[[x]]]
    })
    names(out2) <- names(cgroups)
    out2
}

#' Expressions of human genes.
#'
#' A dataset containing gene expressions of 300 human genes
#' randomly chosen from a list returned by function
#' \code{prepareData(RTCGA = TRUE, cohorts = c('leukemia', 'colon', 'bladder'))}.
#'
#' @format A MultiAssayExperiment object of 3 listed
#' experiments with user-defined names and respective classes.
#' Containing an ExperimentList class object of length 3:
#' [1] leukemia: matrix with 300 rows and 173 columns
#' [2] colon: matrix with 300 rows and 190 columns
#' [3] bladder: matrix with 300 rows and 122 columns.
#'
#' @return data

"exrtcga"

# rtcga <- prepareData(RTCGA = TRUE, cohorts = c('leukemia','colon','bladder'))
# set.seed(5)
# n <- sort(unique(ceiling(runif(306, 0, 8546))))
# exrtcga <- lapply(rtcga, function(x) x[n, ])
# exanova <- aovTopTest(minirtcga, top = 50, sig.level = 0.1,
#  parallel = FALSE)
# extukey <- groupByTukey(exanova, parallel = FALSE, 0.1)
# geneUniverse <- rownames(minirtcga[[1]])
# exallgoanova <- findAllGOs(geneUniverse, extukey, min = 10,
# max = 200, parallel = FALSE, onto = c('MF', 'BP', 'CC')),
# cluster.results <- clustering(exanova,
#   clust.metric = "euclidean", clust.method = "centroid")
# exallgoclust <- findAllGOs(geneUniverse, cluster.results, min = 10,
#   max = 200, parallel = FALSE, onto = c('MF', 'BP', 'CC'))

#' Print groups of clustered genes.
#' @noRd
#' @param x An object of class \code{genecluster}.
#' @param ... Additional arguments to be passed (not in use).
#'
#' @examples
#' unbundleCluster(clustering(exanova,
#'  clust.metric = 'manhattan', clust.method = 'average'))
#'
#' @return genecluster
#'

print.genecluster <- function (x, ...)
{
    y <- x$groups
    y
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

fisher_test <- function(a, b, c, d) {
    .Call('GOpro_fisher_test', PACKAGE = 'GOpro', a, b, c, d)
}

#' Find all significantly different genes based on their
#'  normalized expressions' values.
#' @noRd
#' @param expr A list of data frames of gene expressions
#'  with genes in rows and samples in columns.
#'  Rows must be named with genes' aliases.
#'  The order of samples and genes has to be the same for each data frame.
#' @param top A maximum number of genes to be returned for each group.
#' @param sig.level A numeric value, a significance level used in BH
#'  correction for multiple testing. The default value is 0.05.
#' @param parallel A logical value indicating if a task should
#'  be run on more than one core.
#'  The default value is \code{FALSE}.
#'
#' @return A list of significantly different genes and values
#'  of their expressions.
#'
#' @examples
#' aovTopTest(exrtcga, top = 50, sig.level = 0.1, parallel = FALSE)
#'
#'
#' @import doParallel
#' @import foreach
#' @import parallel

aovTopTest <- function(expr, top, sig.level = 0.05, parallel = FALSE) {

    aovCore <- function(z, expr, group) {
        ex <- unlist(sapply(expr, function(x) x[z, ]))
        gene <- cbind.data.frame(ex, group)
        if (length(ex) == length(unique(group))) {
            warning(paste0("The number of observations within
                           groups is too small in the row number ", z, "."))
            return(NA)
        }
        # at least 2 groups with at least 10% of
        # observations in the smallest group
        if (sum((tapply(gene$ex, gene$group, length) -
                 tapply(gene$ex, gene$group, function(x)
                     sum(is.na(x))) < 0.1 *
                 min(tapply(gene$ex, gene$group, length)))) >
            (length(expr) - 2)) {
            warning("AOV test: too many missing values.")
            NA
        } else {
            (summary(aov(ex ~ group, gene, na.action = na.omit))[[1]][[5]])[1]
        }
    }

    aovTest <- function(expr, parallel) {
        if (is.null(names(expr)) | any(names(expr) == "")) {
            group.names <- paste0("group", seq_along(expr))
        } else {
            group.names <- names(expr)
        }
        nc <- sapply(expr, ncol)
        if (is.null(nc)) {
            stop("The data are insufficient.
                 Provide observations for more than one gene.")
        }
        group <- rep(group.names, nc)
        z <- 0
        if (parallel == TRUE) {
            cores <- detectCores() - 1
            cl <- makeCluster(cores)
            registerDoParallel(cl)
            out_a <- foreach(z = seq_len(nrow(expr[[1]])),
                             .export = "aovCore") %dopar%
                aovCore(z, expr, group)
            stopCluster(cl)
            out_a

        } else {
            foreach(z = seq_len(nrow(expr[[1]]))) %do%
                aovCore(z, expr, group)
        }
    }

    findGeneNames <- function(expr, parallel) {
        if (any(sapply(expr, function(x) is.null(rownames(x)))) ||
            !all(sapply(expr, function(x)
                sapply(expr, function(y) rownames(x) == rownames(y))))) {
            stop("Not all genes are named or not all genes
                 are in the same order for all data frames,
                 or some genes are missing.")
        }
        if (missing(expr) | (is.list(expr) & !is.null(dim(expr))) |
            (length(expr) < 2)) {
            stop("At least two groups are needed for this comparison.")
        }
        if (!is.numeric(unlist(expr))) {
            stop("expr can only contain numbers.")
        }
        if (!is.logical(parallel)) {
            parallel <- FALSE
        }
        anova.result.temp <- aovTest(expr, parallel)
        if (any(is.na(unlist(anova.result.temp)))) {
            anova.result <- anova.result.temp[-which(
                is.na(unlist(anova.result.temp)))]
            names(anova.result) <- row.names(expr[[1]][-which(
                is.na(unlist(anova.result.temp))), ])
        } else {
            anova.result <- anova.result.temp
            names(anova.result) <- row.names(expr[[1]])
        }
        anova.result
    }

    # sig.level and top parameters checking
    if (!is.numeric(sig.level) | (sig.level < 0)) {
        sig.level <- 0.05
        warning("No correct significance level specified.
                Taking 0.05 as the sig.level value.")
    }
    if (length(top) > 1) {
        top <- top[1]
        warning(paste("Too many values provided with the top parameter.
                      Taking ", top, " as the top value."))
    }
    if (top%%1 == 0) {
        top <- round(top)
    }
    if (missing(top) | is.null(top) | (top < 1)) {
        top <- nrow(expr[[1]])
        warning(paste("top parameter is missing or is not positive, taking ",
                      top, " as the top value."))
        top <- nrow(expr[[1]])
    }

    is.equal <- function(top) {
        if (top == sum(p.adj < sig.level)) {
            return(top)
        }
        if (top > sum(p.adj < sig.level)) {
            return(sum(p.adj < sig.level))
        }
        if (sorted[top] == sorted[top + 1]) {
            top <- top + 1
            top <- is.equal(top)
        }
        top
    }

    anova.result <- na.omit(findGeneNames(expr, parallel))
    p.adj <- p.adjust(anova.result, method = "fdr")
    sorted <- sort(p.adj[p.adj < sig.level])
    diff.expressed <- which(row.names(expr[[1]]) %in%
                                names(sorted[seq_len(is.equal(top))]))
    if (length(sorted) == 0) {
        message("Found no significantly different observations.")
    } else {
        lapply(expr, function(x) x[diff.expressed, ])
    }
}

#' Constructor function for the class \code{GO}.
#' @noRd
#' @param allGO A list of lists of vectors each with a p-value
#'  obtained from Fisher test, GO id and all members of each group.
#' @param topGO A list of lists with gene ontology terms
#'  identifiers characteristic for each group.
#'
#' @return An object of class \code{GO}.
#'
#' @examples
#' clustered <- clustering(exanova, clust.metric = 'manhattan',
#'  clust.method = 'average')
#' genecl <- unbundleCluster(clustered)
#' # find all and then top GO terms
#' # geneUniverse is a universe of all genes of interest
#' geneUniverse <- rownames(exrtcga[[1]])
#' all.gos <- findAllGOs(geneUniverse, clustered,
#'  min = 10, max = 200)
#' top.gos <- findTopGOs(all.gos, sig.level = 0.05, top = 2)
#' # create an object of class \code{GO}
#' GO(all.gos, top.gos)
#' @importFrom IRanges CharacterList IntegerList
#' @importFrom S4Vectors DataFrame List


GO <- function(allGO, topGO) {
    out <- DataFrame(allGO = List(lapply(allGO, CharacterList)), topGO = NumericList(topGO))
    invisible(out)
}

#' Print grouped genes with the information of
#'  the labels and GO term's IDs.
#' @noRd
#' @param x An object of class \code{GO}.
#' @param ... Additional arguments to be passed (not in use).
#'
#' @return A data frame with labels, gene's aliases
#'  and GO terms for each group.
#'
#' @examples
#' clustered <- clustering(exanova)
#' genecl <- unbundleCluster(clustered)
#' # find all and then top GO terms
#' # geneUniverse is a universe of all genes of interest
#' geneUniverse <- rownames(exrtcga[[1]])
#' all.gos <- findAllGOs(geneUniverse, clustered,
#'  max = 300, onto = c('MF', 'BP'))
#' top.gos <- findTopGOs(all.gos, sig.level = 0.05, top = 3)
#' # create an object of class \code{GO}
#' printout <- GO(all.gos, top.gos)
#'

printGO <- function(x, ...) {
    allGO <- x[[1]]
    topGO <- x[[2]]
    y <- cbind(profile = names(topGO), GOs = lapply(topGO, names),
               p.values = lapply(topGO, function(x) round(x, 3)),
               GENES = sapply(allGO,
                              function(x) {
                                  if (length(x) == 0) {
                                      NA
                                  } else {
                                      paste0(x[[1]][3:length(x[[1]])],
                                             collapse = " ")
                                  }
                              }))
    y
}

#' Create output.
#' @noRd
#' @param GO An object returned by \code{GO} function.
#'
#' @return An S4 object.
#'

output <- function(GO){
    printedGO <- printGO(GO)
    DataFrame(printedGO)
}

#' Plot gene dendrogram with labels.
#' @noRd
#' @param x an object of class \code{geneclusterplot}.
#' @param over.represented a logical value, if \code{TRUE} then
#'  over represented GO categories are coded with colors for each branch.
#'
#' @return n
#'
#' @examples
#' clustered <- clustering(exanova,
#'  clust.metric = 'manhattan', clust.method = 'average')
#' genecl <- unbundleCluster(clustered)
#' # find all and then top GO terms
#' # geneUniverse is a universe of all genes of interest
#' geneUniverse <- rownames(exrtcga[[1]])
#' all.gos <- findAllGOs(geneUniverse, clustered)
#' top.gos <- findTopGOs(all.gos, sig.level = 0.05, top = 3)
#' # create an object of class \code{GO}
#' printout <- GO(all.gos, top.gos)
#' plotg(geneclusterplot(printout, genecl, top.gos),
#'  over.represented = TRUE)
#'
#'
#' @import dendextend

plotg <- function(printout, genecl, TopGOs, over.represented) {
    hc <- genecl$hclust
    dendro <- as.dendrogram(hc)
    nodes <- get_nodes_xy(dendro)
    roots <- which(nodes[, 2] == 0)
    new.order <- printout[, 1]
    new.order[roots] <- printout[1:length(roots),
                                 1]
    sorted <- sort(nodes[which(nodes[, 2] > 0),
                         2])
    leaves <- sapply(seq_len(length(roots) - 1),
                     function(x) which(nodes[, 2] == sorted[x]))
    new.order[leaves] <- printout[(length(roots) + 1):nrow(nodes), 1]

    # find dominant GO term (MF, CC, BP)
    if (over.represented == TRUE) {
        extendedGO <- extendGO(TopGOs)
        overrep <- ontologyOver(extendedGO)
        names(overrep) <- paste0("G", 1:nrow(nodes))

        bp <- names(which(overrep == "BP"))
        cc <- names(which(overrep == "CC"))
        mf <- names(which(overrep == "MF"))
        both <- overrep[sapply(overrep, length) == 2]
        bpmf <- names(both[which(apply
                                 ((sapply(seq_along(both),
                                          function(x) both[[x]] %in%
                                              c("BP", "MF"))),
                                 2, sum) == 2)])
        bpcc <- names(both[which(apply
                                 ((sapply(seq_along(both),
                                          function(x) both[[x]] %in%
                                              c("BP", "CC"))),
                                 2, sum) == 2)])
        ccmf <- names(both[which(apply
                                 ((sapply(seq_along(both),
                                          function(x) both[[x]] %in%
                                              c("CC", "MF"))),
                                 2, sum) == 2)])
        bpccmf <- names(overrep[sapply(overrep, length) == 3])
        funColors <- rep(1, (1 + nrow(printout)))
        funColors[which(as.character(new.order) %in%
                            bp)] <- 4
        funColors[which(as.character(new.order) %in%
                            cc)] <- 3
        funColors[which(as.character(new.order) %in%
                            mf)] <- 2
        funColors[which(as.character(new.order) %in%
                            bpmf)] <- "cyan4"
        funColors[which(as.character(new.order) %in%
                            ccmf)] <- 6
        funColors[which(as.character(new.order) %in%
                            bpcc)] <- "orchid4"
        funColors[which(as.character(new.order) %in%
                            bpccmf)] <- "goldenrod1"

        dendro <- assign_values_to_branches_edgePar(dend = dendro,
                                                                value = funColors, edgePar = "col")
        dendro %>% set("nodes_pch", 19) %>% set("nodes_col", "cadetblue") %>%
            set("leaves_pch", 19) %>% set("leaves_col", "coral2") %>%
            plot(main = "Dendrogram of differently expressed genes",
                 cex.main = 0.9, ylim = c(0, 1.2 * max(nodes[, 2])),
                 xlim = c(0, max(nodes[, 1])), ylab = "Height")
        text(nodes[, 1], nodes[, 2] + 0.02 * max(nodes[, 2]),
             label = new.order, cex = 0.7,
             pos = 3, srt = 90, font = 2)
        legend(title = "Dominant GO term", x = max(nodes[, 1]),
               y = max(nodes[, 2]), legend = c(" BP", " CC", " MF", " BP, MF",
                                               " BP, CC", " CC, MF",
                                               " BP, CC, MF", " No function"),
               xjust = 1, yjust = 0.5, x.intersp = 0.01,
               y.intersp = 0.8, cex = 0.7, lty = 1,
               col = c(4, 3, 2, "cyan4", 6, "orchid4",
                       "goldenrod1", 1))
    } else {
        dendro <- assign_values_to_branches_edgePar(dend = dendro,
                                                                value = 1,
                                                                edgePar = "col")
        dendro %>% set("nodes_pch", 19) %>% set("nodes_col", "cadetblue") %>%
            set("leaves_pch", 19) %>%
            set("leaves_col", "coral2") %>%
            plot(main = "Dendrogram of differently expressed genes",
                 cex.main = 0.9,
                 ylim = c(0, 1.2 * max(nodes[, 2])),
                 xlim = c(0, max(nodes[, 1])),
                 ylab = "Height")
        text(nodes[, 1], nodes[, 2] + 0.02 * max(nodes[, 2]),
             label = new.order, cex = 0.7,
             pos = 3, srt = 90, font = 2)
    }
    invisible()
}

#' Create profiles names of Tukey test results.
#' @noRd
#' @param part.tukey.res A vector of a list of
#' a result returned by \code{TukeyHSDTest}.
#' @param sig.level A numeric significance level of a test.
#' @param n A number of compared groups.
#'
#' @return A character vector of a name denoting
#' results of all pairwise Tukey comparisons.

makeNames <- function(part.tukey.res, sig.level, n) {
    means <- part.tukey.res[1:n]
    pvals <- part.tukey.res[(n + 1):length(part.tukey.res)]
    names <- names(pvals)

    if (IsConflict(part.tukey.res, sig.level, n)) {
        signedNames <- sapply(names, function(x) {
            temp <- unlist(strsplit(x, "-"))
            means_temp <- sapply(temp, function(z) {
                means[which(names(means) == z)]
            })
            if (pvals[which(names == x)] > sig.level) {
                sign <- "="
            } else {
                sign <- ifelse(means_temp[1] > means_temp[2],
                               ">", "<")
            }
            paste(temp, collapse = sign)
        })
        paste(c(paste(c(signedNames, "ascending means: "), collapse = ";"),
                paste(names(sort(means)), collapse = ",")), collapse = "")
    } else {
        name <- names(means)[1]
        whichpairs <- pvals[sapply(names, function(x) {
            temp <- unlist(strsplit(x, "-"))
            if (temp[1] == name || temp[2] == name) {
                TRUE
            } else {
                FALSE
            }
        })]

        relations <- sapply(names(whichpairs), function(x) {
            if (whichpairs[which(names(whichpairs) == x)] > sig.level) {
                return("equal")
            }
            temp <- unlist(strsplit(x, "-"))
            nameInd <- which(temp == name)
            secondInd <- nameInd%%2 + 1
            tempName1 <- temp[nameInd]
            tempName2 <- temp[secondInd]
            mean1 <- means[which(names(means) == tempName1)]
            mean2 <- means[which(names(means) == tempName2)]
            if (mean1 > mean2) {
                "lower"
            } else {
                "greater"
            }
        })
        lowers <- whichpairs[which(relations == "lower")]
        greaters <- whichpairs[which(relations == "greater")]
        equals <- whichpairs[which(relations == "equal")]

        equalnames <- unique(unlist(strsplit(names(equals), "-")))
        lowernames <- unique(unlist(strsplit(names(lowers), "-")))
        lowernames <- lowernames[-which(lowernames == name)]
        greaternames <- unique(unlist(strsplit(names(greaters),
                                               "-")))
        greaternames <- greaternames[-which(greaternames == name)]
        if (length(equals) > 0) {
            equalmeans <- sort(means[sapply(names(means), function(x) {
                if (length(which(equalnames == x)) == 0) {
                    FALSE
                } else {
                    TRUE
                }
            })])
            output <- paste(names(equalmeans), collapse = "=")
        } else {
            output <- name
        }
        if (length(greaternames) == 1) {
            output <- paste(c(output, greaternames), collapse = "<")
        } else if (length(greaternames) > 1) {
            greatermeans <- means[sapply(names(means), function(x) {
                if (length(which(greaternames == x)) == 0) {
                    FALSE
                } else {
                    TRUE
                }
            })]

            greaterpvals <- sapply(names(pvals), function(x) {
                temp <- unlist(strsplit(x, "-"))
                if (length(which(greaternames == temp[1])) ==
                    0 || length(which(greaternames == temp[2])) ==
                    0) {
                    FALSE
                } else {
                    TRUE
                }
            })
            output <- paste(c(output, makeNames(c(greatermeans,
                                                  pvals[greaterpvals]),
                                                sig.level,
                                                length(greatermeans))),
                            collapse = "<")
        }
        if (length(lowernames) == 1) {
            output <- paste(c(lowernames, output), collapse = "<")
        } else if (length(lowernames) > 1) {
            lowermeans <- means[sapply(names(means), function(x) {
                if (length(which(lowernames == x)) == 0) {
                    FALSE
                } else {
                    TRUE
                }
            })]

            lowerpvals <- sapply(names(pvals), function(x) {
                temp <- unlist(strsplit(x, "-"))
                if (length(which(lowernames == temp[1])) == 0 ||
                    length(which(lowernames == temp[2])) == 0) {
                    FALSE
                } else {
                    TRUE
                }
            })
            output <- paste(c(makeNames(c(lowermeans,
                                          pvals[lowerpvals]),
                                        sig.level,
                                        length(lowermeans)),
                              output),
                            collapse = "<")
        }
        output
    }
}

#' Check if there is a conflict between results of the Tukey test,
#' ex. a=b, b=c, c!=a.
#' @noRd
#' @param part.tukey.res A vector of a list of a result
#' returned by \code{TukeyHSDTest}.
#' @param sig.level A numeric significance level of a test.
#' @param n A number of compared groups.
#'
#' @return A logical value.

IsConflict <- function(part.tukey.res, sig.level,
                       n) {
    means <- part.tukey.res[1:n]
    pvals <- part.tukey.res[(n + 1):length(part.tukey.res)]

    for (i in 1:(n - 1)) {
        group1 <- names(means)[i]
        for (j in (i + 1):n) {
            group2 <- names(means)[j]
            pair_str1 <- paste(c(group1,
                                 group2), collapse = "-")
            pair_str2 <- paste(c(group2,
                                 group1), collapse = "-")
            pair_position <- max(which(names(pvals) == pair_str1),
                                 which(names(pvals) == pair_str2))

            mean1 <- means[which(names(means) == group1)]
            mean2 <- means[which(names(means) == group2)]
            for (k in 1:n) {
                if (i != k && j != k) {
                    group3 <- names(means)[k]
                    mean3 <- means[which(names(means) == group3)]

                    pair_str1 <- paste(c(group1, group3), collapse = "-")
                    pair_str2 <- paste(c(group3, group1), collapse = "-")

                    pair_position2 <- max(which(names(pvals) == pair_str1),
                                          which(names(pvals) == pair_str2))

                    pair_str1 <- paste(c(group2, group3), collapse = "-")
                    pair_str2 <- paste(c(group3, group2), collapse = "-")

                    pair_position3 <- max(which(names(pvals) == pair_str1),
                                          which(names(pvals) == pair_str2))

                    if ((pvals[pair_position] > sig.level)) {
                        if ((pvals[pair_position2] < sig.level &&
                             pvals[pair_position3] > sig.level) ||
                            (pvals[pair_position2] > sig.level &&
                             pvals[pair_position3] < sig.level) ||
                            ((pvals[pair_position2] < sig.level &&
                              pvals[pair_position3] < sig.level) &&
                             ((mean1 > mean3 && mean2 < mean3) ||
                             (mean1 < mean3 && mean2 > mean3)))) {
                            return(TRUE)
                        }
                    }
                }
            }
        }
    }
    FALSE
}

#' Genes grouped by profiles determined by the result
#' returned from \code{tukeyHSDTest} function.
#' @noRd
#' @param expr A list of data frames obtained from \code{aovTopTest}.
#' @param tukey.results A data frame returned by \code{TukeyHSDTest}.
#' @param sig.level A numeric value, a significance level used in
#' Tukey's all pairwise comparisons (\code{groupByTukey}).
#'
#' @return A list of groups of genes named by profiles.

groupByProfiles <- function(expr, tukey.results, sig.level) {
    n <- length(expr)
    named.profiles <- sapply(tukey.results, function(x)
        makeNames(x, sig.level, n))
    groups.names <- sort(unique(named.profiles))
    result.groups <- lapply(groups.names, function(x)
        row.names(expr[[1]][which(named.profiles == x), ]))
    names(result.groups) <- groups.names
    result.groups
}

#' Core function for calculating p-values across all groups of genes
#' using Tukey's Honest Distance test.
#' @noRd
#' @param expr A list of data frames of gene expressions
#'  with genes in rows and samples in columns.
#'  the names of rows represent genes' aliases.
#'  The order of genes has to be the same for each data frame.
#' @param z A number of evaluation of the function needed for
#'  \code{foreach} package.
#' @param group A character name of a group of genes (a cohort).
#'
#' @return A named vector of concatenated: mean value of observations
#'  by groups and the p-value of the Tukey HSD test.

TukeyCore <- function(z, expr, group) {
    n <- length(expr)
    ex <- unlist(sapply(expr, function(x) x[z, ]))
    gene <- cbind.data.frame(ex, group)
    # needed information exactly about all groups
    if (any(sum(tapply(gene$ex, gene$group, length) -
                tapply(gene$ex, gene$group,
                       function(x) sum(is.na(x))) == 0))) {
        rep(NA, n + (n * n - n)/2)
        # a number of groups for which a mean value would be
        # computed plus a number of all possible comparisons in Tukey's test
    } else {
        gene <- na.omit(gene)
        testing <- TukeyHSD(aov(ex ~ group, data = gene))
        mean.val <- tapply(gene$ex, gene$group,
                           mean)
        p.val <- testing$group[seq_len((n * n - n)/2), "p adj"]
        result <- c(mean.val, p.val)
        if (n == 2) {
            names(result) <- "GROUP1-GROUP2"
        }
        result
    }
}

#' Call the calculation of p-values across all groups of genes
#' using Tukey's Honest Distance test optionally on more than
#' one core (actually \code{#(available cores) - 1}).
#' @noRd
#' @param expr A list of data frames of gene expressions with
#'  genes in rows and samples in columns.
#'  The names of rows represent genes' aliases.
#'  The order of genes has to be the same for each data frame.
#' @param parallel A logical value indicating if a task should be
#'  run on more than one core. The default is \code{FALSE}.
#'
#' @return A list of \code{c(mean.val, p.val)}.
#'
#' @import doParallel

tukeyHSDTest <- function(expr, parallel = FALSE) {
    group.names <- names(expr)
    nc <- sapply(expr, ncol)
    if (is.null(nc)) {
        stop("The data are insufficient.
             Provide observations for more than one gene.")
    }
    group <- rep(group.names, nc)
    z <- 0
    if (parallel == TRUE) {
        cores <- detectCores() - 1
        cl <- makeCluster(cores)
        registerDoParallel(cl)
        out <- foreach(z = seq_len(nrow(expr[[1]])),
                       .export = "TukeyCore") %dopar% TukeyCore(z, expr, group)
        stopCluster(cl)
        out
    } else {
        out <- foreach(z = seq_len(nrow(expr[[1]]))) %do%
            TukeyCore(z, expr, group)
    }
    out
}

#' Find top gene ontology terms for groups of genes
#'  based on the result obtained from \code{findAllGOs}.
#' @noRd
#' @param AllGOs A list of lists of vectors each with a p-value
#'  obtained from Fisher test, GO id and all members of each group.
#' @param sig.level A numeric value, a significance
#'  level used in BH correction for multiple testing.
#' @param top A number of the most characteristic
#'  functions of groups of genes to be returned.
#'
#' @return A list of lists of corrected p-values of Fisher test of
#'  characteristic for each group GO terms named with
#'  gene ontology terms identifiers.
#'
#' @examples
#' tukey.results <- groupByTukey(exanova, parallel = FALSE, 0.1)
#' #geneUniverse is a universe of all genes of interest
#' geneUniverse <- rownames(exrtcga[[1]])
#' all.gos <- findAllGOs(geneUniverse, tukey.results,
#'  min = 5, max = 500)
#' findTopGOs(all.gos, sig.level = 0.05, top = 10)
#'

findTopGOs <- function(AllGOs, sig.level, top) {
    lapply(AllGOs, function(group.no) {
        if (length(group.no) == 0) {
            return(NA)
        }
        p.val <- sapply(group.no, function(p.no) {
            record <- p.no
            if (length(record) == 0) {
                return(NA)
            }
            if (is.na(record[1])) {
                return(NA)
            }
            p.val <- record[1]
            if (length(p.val) == length(record[1])) {
                names(p.val) <- record[2]
            } else {
                if (length(p.val) == 0)
                    return(NA) else
                        names(p.val) <- record[2, which(!is.na(record[1]))]
            }
            p.val
        })

        p.val <- na.omit(p.val)
        p.adj <- p.adjust(p.val, "fdr")
        if (length(p.adj) == 0) {
            return(NA)
        }
        if (min(p.adj) > sig.level) {
            return(NA)
        }

        noOfGOs <- top
        if (sum(p.adj < sig.level) < top) {
            noOfGOs <- sum(p.adj < sig.level)
        }
        sorted <- sort(p.adj[p.adj < sig.level])

        is.equal <- function(noOfGOs) {
            if (noOfGOs == sum(p.adj < sig.level)) {
                return(noOfGOs)
            }
            if (sorted[noOfGOs] == sorted[noOfGOs + 1]) {
                noOfGOs <- noOfGOs + 1
                noOfGOs <- is.equal(noOfGOs)
            }
            noOfGOs
        }
        most.char <- sorted[1:is.equal(noOfGOs)]
        most.char
    })
}

#' Find top Gene Ontology terms for given genes.
#'
#' @param groups a MultiAssayExperiment object
#' containing an ExperimentList class object representing
#' gene expressions for at least 3 cohorts.
#'  Rows must be named with genes' aliases.
#'  The order of samples and genes has to be the same for each
#'  ExperimentList class object.
#' @param topAOV A numeric value, a number of most
#'  significantly differentiated genes to be returned.
#' @param sig.levelAOV a numeric value, a significance level
#'  used in BH correction for multiple testing (\code{aovTopTest}).
#' @param sig.levelGO A numeric value, a significance level used
#'  in BH correction for multiple testing (\code{findTopGOs}).
#' @param minGO A minimum number of functions that a gene
#'  needs to represent to be considered as frequent.
#' @param maxGO A maximum number of functions that a gene
#'  needs to represent to be considered as frequent.
#' @param parallel A logical value indicating if a task
#'  should be run on more than one core.
#' @param clust.metric The method to calculate a distance measure
#'  used in hierarchical clustering, possible names: "euclidean",
#'  "maximum", "manhattan", "canberra", "binary" or "minkowski".
#' @param clust.method The agglomeration method used to cluster genes.
#'  This should be #'one of "ward.D",
#'  "ward.D2", "single", "complete", "average" (= UPGMA),
#'  "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
#' @param topGO A number of the most characteristic functions
#'  of groups of genes to be returned.
#' @param grouped A method of grouping genes, one of 'tukey' and 'clustering'.
#' @param dist.matrix A matrix with calculated distances to be
#'  used as a metric by \code{hclust} function.
#' @param sig.levelTUK A numeric value, a significance level used in
#'  Tukey's all pairwise comparison (\code{groupByTukey}).
#' @param onto An ontology or ontologies to be searched
#'  for significant GO terms,
#'  at least one of 'MF' (molecular function),
#'  'BP' (biological process), and 'CC' (cellular component).
#' @param extend A logical value indicating if an extended
#'  version of the output should be presented.
#' @param over.rep A logical value indicating if an over
#'  represented GO terms should be presented in the plot.
#'
#' @return A data frame containing the top gene ontology terms for
#'  each group of genes and the gene aliases.
#'
#' @examples
#' findGO(exrtcga, grouped = 'clustering', topGO = 10, onto = 'MF')
#' findGO(exrtcga, grouped = 'tukey', topGO = 2, extend = TRUE)
#' @export
#'
#' @useDynLib GOpro, .registration=TRUE
#'
#' @importFrom Rcpp sourceCpp
#' @importFrom MultiAssayExperiment assay experiments

findGO <- function(groups, topAOV = 50, sig.levelAOV = 0.05,
                   parallel = FALSE, grouped = "tukey",
                   sig.levelGO = 0.05, minGO = 5, maxGO = 500,
                   clust.metric = NULL, clust.method = NULL,
                   dist.matrix = NULL, topGO = 3, sig.levelTUK = 0.05,
                   onto = c("MF", "BP", "CC"), extend = FALSE, over.rep = FALSE) {
    #groups <- assay(experiments(groups))
    g2 <- experiments(groups)
    groups <- sapply(g2, assay)
    aov.results <- aovTopTest(groups, topAOV, sig.levelAOV, parallel)
    geneUniverse <- rownames(groups[[1]])
    if (!(grouped == "tukey" | grouped == "clustering")) {
        stop(paste("There is no grouping method called", grouped))
    }
    if (grouped == "tukey") {
        tukey.results <- groupByTukey(aov.results, parallel, sig.levelTUK)
        AllGOs <- findAllGOs(geneUniverse, tukey.results,
                             onto, minGO, maxGO, parallel)
        TopGOs <- findTopGOs(AllGOs, sig.levelGO, topGO)
        if (extend == TRUE) {
            out <- extendoutput(TopGOs)
        } else {
            out <- output(GO(AllGOs, TopGOs))
        }
    }
    if (grouped == "clustering") {
        cluster.results <- clustering(aov.results,
                                      clust.metric, clust.method, dist.matrix)
        all.gos <- findAllGOs(cluster.results,
                              geneUni = rownames(groups[[1]]))
        top.gos <- findTopGOs(all.gos, sig.levelGO, topGO)
        unbundled <- unbundleCluster(cluster.results)
        printout <- printGO(GO(all.gos, top.gos))
        plotg(printout, unbundled, top.gos,
              over.represented = over.rep)
        if (extend == TRUE) {
            out <- extendoutput(top.gos)
        } else {
            out <- output(GO(all.gos, top.gos))
        }
    }
    out
}

#' Determine group of genes by all pairwise comparisons (Tukey's method).
#' @noRd
#' @param expr A named list of data frames of significantly
#'  different gene expressions
#'  with genes in rows and samples in columns.
#'  Rows should be named with genes' aliases.
#'  The order of genes has to be the same for each data frame.
#'  The names of list elements should refer to groups of different cases.
#' @param parallel A logical value indicating if a task
#'  should be run on more than one core.
#'  The default value is \code{FALSE}.
#' @param sig.level A numeric value, a significance level used in BH correction
#'  for multiple testing.
#'  The default value is 0.05.
#'
#' @return A data frame with groups of genes named by profiles.
#'  Profiles are determined by the differences between groups.
#'
#' @examples
#' groupByTukey(exanova, sig.level = 0.1)
#'

groupByTukey <- function(expr, parallel = FALSE, sig.level = 0.05) {
    if (!all(sapply(expr, function(x)
        sapply(expr, function(y) row.names(x) == row.names(y))))) {
        stop("Not all genes are named or not all genes
            are in the same order for all data frames,
            or some genes are missing.")
    }
    if (missing(expr) | (is.list(expr) & !is.null(dim(expr))) |
        (length(expr) < 2)) {
        stop("At least two groups are needed for this comparison.")
    }
    if (is.null(names(expr)) | any(names(expr) == "")) {
        names(expr) <- paste0("group", seq_along(expr))
        warning(paste0("Not all elements of the list were named."))
    }
    if (!is.numeric(unlist(expr))) {
        stop("expr can only contain numbers.")
    }
    if (!is.numeric(sig.level) | (sig.level < 0)) {
        sig.level <- 0.05
        warning("No correct significance level specified.
                Taking 0.05 as the sig.level value.")
    }
    tukey.results <- tukeyHSDTest(expr, parallel)
    groupByProfiles(expr, tukey.results, sig.level)
    }

---
title: "GOpro: Determine groups of genes and find their most 
    characteristic GO term"
author: "Lidia Chrabaszcz"
output: BiocStyle::html_document
vignette: >
  %\VignetteEncoding{UTF-8}
  %\VignetteIndexEntry{GOpro: Determine groups of genes and find their characteristic GO term}
  %\VignetteEngine{knitr::rmarkdown}
---
```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

# Installation
```{r, eval = FALSE}
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("GOpro", dependencies = TRUE)
```

# Loading
After the package is installed, it can be loaded into R workspace typing
```{r, eval=TRUE, results='hide', warning=FALSE, message=FALSE}
library(GOpro)
```

# Overview
This document presents an overview of the GOpro package. 
This package is for determining groups of genes and finding characteristic 
functions for these groups. It allows for interpreting groups of genes by their 
most characteristic biological function. It provides one function *findGO*
which is based on the set of methods. One of these methods allows for determining 
significantly different genes between at least two  distinct groups 
(i.e. patients with different medical condition) - the ANOVA test with 
correction for multiple testing. It also provides two methods for grouping 
genes. One of them is so-called all pairwise comparisons 
utilizing Tukey's method. By this method profiles of genes are determined, 
i.e. in terms of the gene expression genes are grouped according to the 
differences in the expressions between given cohorts. 
Another method of grouping is hierarchical 
clustering. This package provides a method for finding the most characteristic 
gene ontology terms for anteriorly obtained groups using the 
one-sided Fisher's test for overrepresentation of the gene ontology term.
If genes were grouped by the hierarchical clustering, then the most characteristic 
function is 
found for all possible groups (for each node in the dendrogram).


# Details 
Genes must be named with the gene aliases and they must be
arranged in the same order for each cohort.

## Determining significantly different genes based on their expressions
Genes which are statistically differently expressed are selected for the
further analysis by ANOVA test. 
The *topAOV* parameter denotes the maximum number of significantly 
different genes
to be selected. The significance level of ANOVA test is specified 
by the *sig.levelAOV* parameter.  
This threshold is used as the significance level in the BH correction 
for multiple testing.
In the case of equal p-values of the test (below the given threshold), 
all genes for which the p-value of the test is the same as for the gene
numbered with the *topAOV* value are included in the result. 

## Grouping genes based on their similarity
There are two methods provided for grouping genes. They are
specified by the *grouped* parameter. The first one using
Tukey's test is called when *grouped* equals *'tukey'* and the
second one can be called by using the *'clustering'* value.

### All pairwise comparisons by Tukey's test
The Tukey's test is applied to group genes based on their profiles.
The *sig.levelTUK* parameter denotes the significance level of Tukey's test.
For each gene two-sided Tukey's test is conducted among cohorts.
The mean expressions in the cohorts are arranged in 
ascending order and the result of the test is adapted. All genes with
the same order of means and the same result of the test are grouped together.
I.e. notation *colon\=bladder\<leukemia* denotes that the mean expression 
calculated for a 
particular gene in the colon cancer cohort is statistically the same as for the
bladder cancer cohort. Both mean values determined for aforementioned cohorts are
statistically lower than the mean expression measured for the leukemia cohort.

### Hierarchical clustering
Hierarchical clustering is utilized for grouping genes based on dissimilarity
measures. In this case all clusters are subjected to the further analysis.
The *clust.metric* parameter is a method to calculate a distance measure
between genes, the *clust.method* is the agglomeration method used to cluster genes,
and the *dist.matrix* is an optional parameter for distance matrix if available
*clust.metric*
methods are not sufficient for the user.

## Finding characteristic gene ontology terms
For each specified group the *org.Hs.eg.db* is searched for all relevant
GO terms. The number of counts of each GO term is calculated for each group.
Then the Fisher's test is applied in order to find the most characteristic GO terms
for each group of genes.
The *minGO* and *maxGO* parameters denote the range of 
counts of genes annotated with each GO term. All GO terms with 
counts above or below this range are omitted in the analysis. It enables for the
exclusion of very rare or very common gene ontology terms. 
Gene ontology domains to be searched for GO terms can be specified by the *onto*
parameter. Possible domains are: *'MF'* (molecular function),
*'BP'* (biological process), and *'CC'* (cellular component).
The *sig.levelGO* parameter specifies the significance level of the Fisher's test
(correction for multiple testing is included).

# Data

Data used in this example comes from 
[The Cancer Genome Atlas](https://tcga-data.nci.nih.gov/tcga/). 
They were downloaded via RTCGA.PANCAN12 package. The data represents 
expressions of 300 genes randomly chosen from 16115 genes determined 
for each patient (sample). Three cohorts are 
included: acute myleoid leukemia, colon cancer, and bladder cancer.
The data is stored in this *GOpro* package as a MultiAssayExperiment
object.

An example of the data structure:
```{r, cache=TRUE, message=FALSE, results='markup'}
exrtcga
```

# Example
To run the analysis with default parameters on the exrtcga object call:
```{r, results='hold', message=FALSE, cache=TRUE}
findGO(exrtcga)
```

The results of the analysis can be presented in a more descriptive 
way or in a concise one. To get more descriptive results use *extend=TRUE*
option.
Additionally, the *TERM*, *DEFINITION*, and *ONTOLOGY* for each ontology
term are returned.
```{r, results='hold', message=FALSE, cache=TRUE}
findGO(exrtcga, extend = TRUE)
```

In order to find top 2 GO terms for genes grouped by hierarchical clustering 
run the following call. The result of clustering is
presented on the plot.
```{r, results='markup', message=FALSE, cache=TRUE, fig.keep='all', fig.show='hold', dev='png'}
findGO(exrtcga, topGO = 2, grouped = 'clustering')
```
The plot can be also enriched with information about the most frequent
ontology domain for each node on the dendrogram.
```{r, results='markup', message=FALSE, cache=TRUE, fig.keep='all', fig.show='hold', dev='png'}
findGO(exrtcga, topGO = 2, grouped = 'clustering', over.rep = TRUE)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{exrtcga}
\alias{exrtcga}
\title{Expressions of human genes.}
\format{A MultiAssayExperiment object of 3 listed
experiments with user-defined names and respective classes.
Containing an ExperimentList class object of length 3:
[1] leukemia: matrix with 300 rows and 173 columns
[2] colon: matrix with 300 rows and 190 columns
[3] bladder: matrix with 300 rows and 122 columns.}
\usage{
exrtcga
}
\value{
data
}
\description{
A dataset containing gene expressions of 300 human genes
randomly chosen from a list returned by function
\code{prepareData(RTCGA = TRUE, cohorts = c('leukemia', 'colon', 'bladder'))}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOpro.R
\docType{package}
\name{GOpro}
\alias{GOpro}
\alias{GOpro-package}
\title{GOpro: find the most characteristic gene ontology terms for groups of genes}
\description{
Based on the gene expressions find the structure somewhat
comparable to gene signature. From all given genes,
determine which are significantly different between sets.
These sets may relate to different health conditions of patients,
i.e. different types of cancer.
Then divide interesting genes into subsets.
Genes belong to a particular subset if they share the same feature.
There are two implemented methods that can be used to create
genes' subsets.
The first method is so-called all pairwise comparisons by
Tukey's procedure.
Genes that have the same profile (a result of all comparisons)
are assigned to one subset.
The second way of determining subsets is a method of
hierarchical clustering.
When all genes are divided into subsets, then
for each subset all relevant GO terms are searched for
in org.Hs.eg.db database.
Each found GO terms is tested using Fisher's test to find out
which of them are the most characteristic for the given subset of genes.
}
\seealso{
{\link{findGO}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findGO.R
\name{findGO}
\alias{findGO}
\title{Find top Gene Ontology terms for given genes.}
\usage{
findGO(groups, topAOV = 50, sig.levelAOV = 0.05, parallel = FALSE,
  grouped = "tukey", sig.levelGO = 0.05, minGO = 5, maxGO = 500,
  clust.metric = NULL, clust.method = NULL, dist.matrix = NULL,
  topGO = 3, sig.levelTUK = 0.05, onto = c("MF", "BP", "CC"),
  extend = FALSE, over.rep = FALSE)
}
\arguments{
\item{groups}{a MultiAssayExperiment object
containing an ExperimentList class object representing
gene expressions for at least 3 cohorts.
 Rows must be named with genes' aliases.
 The order of samples and genes has to be the same for each
 ExperimentList class object.}

\item{topAOV}{A numeric value, a number of most
significantly differentiated genes to be returned.}

\item{sig.levelAOV}{a numeric value, a significance level
used in BH correction for multiple testing (\code{aovTopTest}).}

\item{parallel}{A logical value indicating if a task
should be run on more than one core.}

\item{grouped}{A method of grouping genes, one of 'tukey' and 'clustering'.}

\item{sig.levelGO}{A numeric value, a significance level used
in BH correction for multiple testing (\code{findTopGOs}).}

\item{minGO}{A minimum number of functions that a gene
needs to represent to be considered as frequent.}

\item{maxGO}{A maximum number of functions that a gene
needs to represent to be considered as frequent.}

\item{clust.metric}{The method to calculate a distance measure
used in hierarchical clustering, possible names: "euclidean",
"maximum", "manhattan", "canberra", "binary" or "minkowski".}

\item{clust.method}{The agglomeration method used to cluster genes.
This should be #'one of "ward.D",
"ward.D2", "single", "complete", "average" (= UPGMA),
"mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).}

\item{dist.matrix}{A matrix with calculated distances to be
used as a metric by \code{hclust} function.}

\item{topGO}{A number of the most characteristic functions
of groups of genes to be returned.}

\item{sig.levelTUK}{A numeric value, a significance level used in
Tukey's all pairwise comparison (\code{groupByTukey}).}

\item{onto}{An ontology or ontologies to be searched
for significant GO terms,
at least one of 'MF' (molecular function),
'BP' (biological process), and 'CC' (cellular component).}

\item{extend}{A logical value indicating if an extended
version of the output should be presented.}

\item{over.rep}{A logical value indicating if an over
represented GO terms should be presented in the plot.}
}
\value{
A data frame containing the top gene ontology terms for
 each group of genes and the gene aliases.
}
\description{
Find top Gene Ontology terms for given genes.
}
\examples{
findGO(exrtcga, grouped = 'clustering', topGO = 10, onto = 'MF')
findGO(exrtcga, grouped = 'tukey', topGO = 2, extend = TRUE)
}
